﻿using Kivii.DataAnnotations;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    public abstract class SettlementBase : EntityWithMetadata
    {
        #region 关联信息 Owner,Offset
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [StringLength(500)]
        public string OwnerName { get; set; }
        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }
        #endregion

        #region 业务关联(结算财务表中,Owner记录所属部门,Biz记录关联业务)

        [ApiMember(Description = "业务Kvid,根据结算精细度,Settlement中的BizKvid可能会和SettlementDetail中的BizKvid一致," +
            "查询时需要同时查询两表数据")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }
        #endregion

        #region 结算双方 Payer需方单位  Payee供方单位
        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付方单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }


        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收方单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }
        #endregion

        #region 结算相关
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public virtual string SerialNumber { get; set; }

        //不使用枚举型，不确定的类型
        [ApiMember(Description = "结算类型")]
        [InternalSetter]
        [StringLength(50), Default("")]
        public string Type { get; set; }

        [ApiMember(Description = "预结算金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPlan { get; set; }

        [ApiMember(Description = "结算金额,实际结算金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "已付金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPayment { get; set; }

        [ApiMember(Description = "支付Kvid")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid PayKvid { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime
        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "组织机构名称,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [StringLength(200), CurrentOrganizationName]
        public string OrganizationName { get; set; }

        [ApiMember(Description ="组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大3000字")]
        [StringLength(3000), Default("")]
        public string Remark { get; set; }
        #endregion

        /// <summary>
        /// 用于创建时 接口提供批量结算时使用
        /// </summary>
        [Ignore]
        public List<SettlementDetail> Details { get; set; }

    }

    /// <summary>
    /// 财务模块中的结算 
    /// </summary>
    [Api(Description = "结算")]
    [Alias(Configs.TableNameSettlement)]
    public class Settlement : SettlementBase, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public override string SerialNumber { get; set; }

        //[ApiMember(Description = "已开票金额")]
        //[InternalSetter]
        //[DecimalLength(15, 2), Default(0)]
        //public decimal AmountInvoice { get; set; }

        private static MethodInfo _realMethod = null;

        public object GetSettlementDetails()
        {
            var rtns = new List<SettlementDetail>();
            Type entityType = Text.AssemblyUtils.FindType(this.BizType);
            if (entityType == null) return rtns;

            if (_realMethod == null)
            {
                var function = new Func<object>(onExecution<Entity>);
                _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
            }
            return _realMethod.MakeGenericMethod(entityType).Invoke(this, null);
        }
        private object onExecution<T>() where T : IEntity
        {
            var rtns = new List<SettlementDetail>();
            var conn = KiviiContext.GetOpenedDbConnection<EntitySettlementDetail<T>>();
            var settlementDetals = conn.Select<EntitySettlementDetail<T>>(o => o.SettlementKvid == this.Kvid);
            if (settlementDetals.IsNullOrEmpty()) return rtns;
            return settlementDetals;
        }
    }

    [Alias(Configs.TableNamePrefixSettlement)]
    public class EntitySettlement<T> : SettlementBase, IEntityInGenericTypeDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public override string SerialNumber { get; set; }
    }
}
