﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class PaymentExtension
    {
        /// <summary>
        /// 获取子付款记录的流水号
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static string GetSubSerialNumber(this Payment payment, IDbConnection conn = null)
        {
            payment.RootKvid.ThrowIfEmpty("RootKvid不能为空");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
            //var rootPayment = conn.SingleById<Payment>(payment.RootKvid);
            //rootPayment.ThrowIfNull("主付款记录不存在");
            var total = conn.Count<Payment>(o => o.RootKvid == payment.RootKvid);
            if (total == 0) return string.Empty;
            //allcount不需要加1，因为Root的也统计进去了
            return $"{payment.SerialNumber}-{total}";
        }

        /// <summary>
        /// 重新计算Payment的 AmountUsed
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountUsed(this Payment payment,IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Payment>();
            }

            //var amountInvoiced = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountInvoice), p => p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            var amountUsed = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountUsed), p => p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            Payment parentPayment = null;
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                if (payment.OffsetKvid == Guid.Empty)
                {
                    //payment.AmountInvoice = amountInvoiced;
                    //payment.AddOnlyProperties(o => o.AmountInvoice);
                    payment.AmountUsed = amountUsed;
                    payment.AddOnlyProperties(o => o.AmountUsed);
                    conn.UpdateOnly(payment);
                }
                if (parentPayment != null) parentPayment.RecalculateAmountUsed(conn);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }

        /// <summary>
        /// 重新计算Payment的 AmountInvoice
        /// 只针对Bank等根级Payment
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Payment RecalculateAmountInvoice(this Payment payment, IDbConnection conn = null)
        {
            payment.ThrowIfNull("要更新的付款记录不能为空!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此到账已作废 无法重新计算开票金额");
            (payment.RootKvid != payment.Kvid).ThrowIfTrue("不支持的类型");
            (payment.Type != PaymentType.Bank).ThrowIfTrue("不支持的类型");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();

            var amountInvoiced = conn.Scalar<Payment, decimal>(o => Sql.Sum(o.AmountInvoice), p => p.Type == PaymentType.Split && p.ParentKvid == payment.Kvid && p.OffsetKvid == Guid.Empty);
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                payment.AmountInvoice = amountInvoiced;
                payment.AddOnlyProperties(o => o.AmountInvoice);
                conn.UpdateOnly(payment);
                trans?.Commit();
                return payment;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }

        /// <summary>
        /// 接受一个从系统外来的新付款记录，收付款账户中增加和减小
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static List<AccountDetail> Accept(this Payment payment)
        {
            payment.ThrowIfNull("付款记录不能为空!");
            (payment.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (payment.PayerAccountKvid == Guid.Empty).ThrowIfTrue("付款账户不能为空!");
            (payment.PayeeAccountKvid == Guid.Empty).ThrowIfTrue("收款账户不能为空!");
            (payment.Amount <= 0).ThrowIfTrue("付款记录发生金额不能少于0!");
            (payment.AmountInvoice == 0).ThrowIfFalse("付款记录开票金额不为0!");
            (payment.AmountUsed == 0).ThrowIfFalse("付款记录使用金额不为0!");
            (payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.Bank).ThrowIfTrue("仅支持接受现金刷卡或者银行流水记录");
            if (payment.Type == PaymentType.Bank) (payment.AmountSplited != 0).ThrowIfTrue("付款记录拆分不为0!");
            else (payment.AmountSplited != payment.Amount).ThrowIfTrue("现金刷卡拆分金额错误!");
            if (payment.ParentKvid != Guid.Empty) payment.ParentKvid = Guid.Empty;
            if (payment.OffsetKvid != Guid.Empty) payment.OffsetKvid = Guid.Empty;
            if (payment.Kvid == Guid.Empty) payment.Kvid = Guid.NewGuid();
            if (payment.RootKvid != payment.Kvid) payment.RootKvid = payment.Kvid;

            var rtns = new List<AccountDetail>();
            #region 付款账户变动
            var payerAccountDetail = new AccountDetail();
            payerAccountDetail.AccountKvid = payment.PayerAccountKvid;
            payerAccountDetail.BizId = payment.SerialNumber;
            payerAccountDetail.BizKvid = payment.Kvid;
            payerAccountDetail.BizType = typeof(Payment).FullName;

            payerAccountDetail.PayerKvid = payment.PayerKvid;
            payerAccountDetail.PayerName = payment.PayerName;
            payerAccountDetail.PayerAccountKvid = payment.PayerAccountKvid;
            payerAccountDetail.PayerAccountName = payment.PayerAccountName;
            payerAccountDetail.PayerAccountSerialNumber = payment.PayerAccountSerialNumber;

            payerAccountDetail.PayeeKvid = payment.PayeeKvid;
            payerAccountDetail.PayeeName = payment.PayeeName;
            payerAccountDetail.PayeeAccountKvid = payment.PayeeAccountKvid;
            payerAccountDetail.PayeeAccountName = payment.PayeeAccountName;
            payerAccountDetail.PayeeAccountSerialNumber = payment.PayeeAccountSerialNumber;

            payerAccountDetail.AmountPayment = payment.Amount;

            payerAccountDetail.Summary = payment.Summary;
            payerAccountDetail.Summary = payment.Summary;
            payerAccountDetail.Remark = payment.Remark;
            rtns.Add(payerAccountDetail);
            #endregion

            #region 收款账户变动
            var payeeAccountDetail = new AccountDetail();
            payeeAccountDetail.AccountKvid = payment.PayeeAccountKvid;
            payeeAccountDetail.BizId = payment.SerialNumber;
            payeeAccountDetail.BizKvid = payment.Kvid;
            payeeAccountDetail.BizType = typeof(Payment).FullName;

            payeeAccountDetail.PayerKvid = payment.PayerKvid;
            payeeAccountDetail.PayerName = payment.PayerName;
            payeeAccountDetail.PayerAccountKvid = payment.PayerAccountKvid;
            payeeAccountDetail.PayerAccountName = payment.PayerAccountName;
            payeeAccountDetail.PayerAccountSerialNumber = payment.PayerAccountSerialNumber;

            payeeAccountDetail.PayeeKvid = payment.PayeeKvid;
            payeeAccountDetail.PayeeName = payment.PayeeName;
            payeeAccountDetail.PayeeAccountKvid = payment.PayeeAccountKvid;
            payeeAccountDetail.PayeeAccountName = payment.PayeeAccountName;
            payeeAccountDetail.PayeeAccountSerialNumber = payment.PayeeAccountSerialNumber;

            payeeAccountDetail.AmountPayment = payment.Amount;

            payeeAccountDetail.Summary = payment.Summary;
            payeeAccountDetail.Summary = payment.Summary;
            payeeAccountDetail.Remark = payment.Remark;
            rtns.Add(payeeAccountDetail);
            #endregion

            return rtns;
        }

        /// <summary>
        /// 到账拆分,并且更新原payment.AmountSplit字段
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="amountSplit"></param>
        /// <param name="ownerKvid"></param>
        /// <param name="ownerName"></param>
        /// <returns></returns>
        public static Payment BizSplit(this Payment payment, decimal amountSplit, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            payment.ThrowIfNull($"传入payment参数为空");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以认领!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可认领!");
            (amountSplit <= 0).ThrowIfTrue("拆分认领金额不能为零或负值。");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可认领!");
            if (payment.AmountSplited + amountSplit > payment.Amount) throw new Exception("认领金额超出范围!");
            decimal amountInvoice = 0;
            if (payment.AmountInvoice > 0)//如果要拆分的到账已经开过票 就要计算拆分项的开票金额
            {
                if (payment.AmountInvoice < payment.Amount)//到账的部分进行了开票 就需要计算拆分项的开票金额
                {
                    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Payment>();
                    //查看已经拆分的项目分得的已经开票金额 
                    var existSplits = conn.Select<Payment>(o => o.RootKvid == payment.RootKvid && o.Type == PaymentType.Split && o.OffsetKvid == Guid.Empty);
                    if (!existSplits.IsNullOrEmpty())
                    {
                        //剩余的开票金额
                        var remainAmountInvoice = payment.AmountInvoice - existSplits.Sum(o => o.AmountInvoice);
                        if (remainAmountInvoice > 0) amountInvoice = remainAmountInvoice > amountSplit ? amountSplit : remainAmountInvoice;
                    }
                }
                else amountInvoice = amountSplit;//否则直接记录拆分项完全开票
            }
            #region 新增拆分项目
            var splitPayment = new Payment();
            splitPayment.PopulateWith(payment);//先从父级中拷贝所有参数
                                               //赋值
            splitPayment.Kvid = Guid.NewGuid();
            splitPayment.ParentKvid = payment.Kvid;
            splitPayment.RootKvid = payment.RootKvid;
            splitPayment.OffsetKvid = Guid.Empty;
            splitPayment.OwnerKvid = ownerKvid == null ? Guid.Empty : ownerKvid.Value;
            splitPayment.OwnerName = ownerName;
            //业务设置
            splitPayment.BizId = string.Empty;
            splitPayment.BizKvid = Guid.Empty;
            splitPayment.BizType = string.Empty;
            splitPayment.SerialNumber = payment.GetSubSerialNumber();
            splitPayment.Type = PaymentType.Split;
            //金额设置
            splitPayment.Amount = amountSplit;
            splitPayment.AmountSplited = 0;
            splitPayment.AmountUsed = 0;
            splitPayment.AmountInvoice = amountInvoice;// payment.AmountInvoice > amountSplit ? amountSplit : payment.AmountInvoice;
            splitPayment.Summary = string.Empty;
            splitPayment.Remark = string.Empty;
            splitPayment.OperateTime = payment.OperateTime;
            splitPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            splitPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            //splitPayment.Status = 1;
            #endregion

            #region 更新主项字段
            payment.AmountSplited += amountSplit;
            payment.AddOnlyProperties(o => o.AmountSplited);
            #endregion

            return splitPayment;
        }

        /// <summary>
        /// 非业务收款拆分处理,非业务收款或退款
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="amountSplit"></param>
        /// <param name="ownerKvid"></param>
        /// <param name="ownerName"></param>
        /// <returns></returns>
        public static Payment UnBizSplit(this Payment payment, decimal amountSplit, PaymentType paymentType)
        {
            payment.ThrowIfNull($"传入payment参数为空");
            if (payment.Type != PaymentType.Bank) throw new Exception("非银行到账不可以操作!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("已冲销收款不可操作!");
            (amountSplit <= 0).ThrowIfTrue("操作金额不能为零或负值。");
            if (payment.AmountSplited == payment.Amount) throw new Exception("无剩余收款可操作!");
            if (payment.AmountSplited + amountSplit > payment.Amount) throw new Exception("操作金额超出范围!");
            if (paymentType != PaymentType.UnBiz && paymentType != PaymentType.Refund) throw new Exception("此方法只支持非业务拆分或退款拆分!");

            #region 新增拆分项目
            var splitPayment = new Payment();
            splitPayment.PopulateWith(payment);//先从父级中拷贝所有参数
                                               //赋值
            splitPayment.Kvid = Guid.NewGuid();
            splitPayment.ParentKvid = payment.Kvid;
            splitPayment.RootKvid = payment.RootKvid;
            splitPayment.OffsetKvid = Guid.Empty;
            splitPayment.OwnerKvid = Guid.Empty;
            splitPayment.OwnerName = string.Empty;
            //业务设置
            splitPayment.BizId = string.Empty;
            splitPayment.BizKvid = Guid.Empty;
            splitPayment.BizType = string.Empty;
            splitPayment.SerialNumber = payment.GetSubSerialNumber();
            splitPayment.Type = paymentType;
            //金额设置
            splitPayment.Amount = amountSplit;
            splitPayment.AmountSplited = 0;
            splitPayment.AmountUsed = 0;
            splitPayment.AmountInvoice = 0;
            splitPayment.Summary = string.Empty;
            splitPayment.Remark = string.Empty;
            splitPayment.OperateTime = payment.OperateTime;
            splitPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            splitPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            //splitPayment.Status = 1;
            #endregion

            #region 更新主项字段
            payment.AmountSplited += amountSplit;
            payment.AddOnlyProperties(o => o.AmountSplited);
            #endregion

            return splitPayment;
        }

        /// <summary>
        /// 到账冲账,此方法不做业务判断,传过来的Payment均可Offset,除已经冲账过的
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Payment Offset(this Payment payment, string remark)
        {
            payment.ThrowIfNull("付款记录不能为空!");
            (payment.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("收项已冲账");

            #region payment冲账
            var newPaymentKvid = Guid.NewGuid();
            payment.OffsetKvid = newPaymentKvid;
            payment.AddOnlyProperties(o => o.OffsetKvid);
            payment.Remark = $"{remark}";
            payment.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetPayment = new Payment();
            offsetPayment.PopulateWith(payment);
            offsetPayment.OffsetKvid = payment.Kvid;//新的冲帐关联用原来的Kvid
            offsetPayment.Kvid = newPaymentKvid;
            offsetPayment.SerialNumber = payment.SerialNumber.IsNullOrEmpty() ? "" : ("-" + payment.SerialNumber);
            offsetPayment.Remark = $"{remark}";
            offsetPayment.Amount = 0 - offsetPayment.Amount;//反向值
            #endregion

            return offsetPayment;
        }

        /// <summary>
        /// 到账转账至账户
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="accountPayee"></param>
        /// <param name="accountPayer"></param>
        /// <param name="amount"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Payment Transfer(this Payment payment, Account accountPayee, Account accountPayer, decimal amount, string remark)
        {
            payment.ThrowIfNull("到账不能为空!");
            accountPayee.ThrowIfNull("账户不能为空!");
            accountPayer.ThrowIfNull("账户不能为空!");
            (amount <= 0).ThrowIfTrue("入账金额不能小于等0!");
            (payment.OffsetKvid != Guid.Empty).ThrowIfTrue("此项已冲账!");
            if (payment.Amount - payment.AmountUsed < amount) throw new Exception($"此到账可使用金额不足,仅剩:{payment.Amount - payment.AmountUsed}元");
            (payment.Currency != accountPayee.Currency && accountPayee.Currency != accountPayer.Currency).ThrowIfTrue("账户的货币单位不一致!");

            var transferPaymentKvid = Guid.NewGuid();
            #region 创建入账Payment
            var transferPayment = new Payment();
            transferPayment.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            transferPayment.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            transferPayment.Kvid = transferPaymentKvid;
            transferPayment.RootKvid = payment.RootKvid;
            transferPayment.ParentKvid = payment.Kvid;
            transferPayment.Type = PaymentType.Transfer;
            transferPayment.BizId = accountPayee.Name;
            transferPayment.BizKvid = accountPayee.Kvid;
            transferPayment.BizType = typeof(Account).FullName;

            transferPayment.PayerName = payment.PayerName;
            transferPayment.PayerKvid = payment.PayerKvid;
            transferPayment.PayerAccountName = accountPayer.Name;
            transferPayment.PayerAccountKvid = accountPayer.Kvid;
            transferPayment.PayerAccountOwnerName = accountPayer.OwnerName;
            transferPayment.PayerAccountSerialNumber = accountPayer.SerialNumber;

            transferPayment.PayeeName = accountPayee.Name;
            transferPayment.PayeeKvid = accountPayee.Kvid;
            transferPayment.PayeeAccountName = accountPayee.Name;
            transferPayment.PayeeAccountKvid = accountPayee.Kvid;
            transferPayment.PayeeAccountOwnerName = accountPayee.OwnerName;
            transferPayment.PayeeAccountSerialNumber = accountPayee.SerialNumber;
            transferPayment.Amount = amount;
            transferPayment.AmountSplited = 0;
            transferPayment.AmountUsed = amount;
            transferPayment.OperateTime = DateTime.Now;
            transferPayment.OperatorName = KiviiContext.CurrentMember.FullName;
            transferPayment.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            transferPayment.Currency = payment.Currency;
            transferPayment.Summary = $"{remark}";
            transferPayment.Remark = $"{remark}";
            #endregion

            payment.AmountUsed += amount;
            payment.AddOnlyProperties(o => o.AmountSplited);

            return transferPayment;
        }

        /// <summary>
        /// Payment与Settlement产生关联的方法
        /// </summary>
        /// <param name="payments"></param>
        /// <param name="settlements"></param>
        /// <returns></returns>
        public static List<Payment> Correlating(this List<Payment> payments,List<Settlement> settlements)
        {
            payments.ThrowIfNullOrEmpty("要关联的Payments不能为空!");
            settlements.ThrowIfNullOrEmpty("要关联的Settlements不能为空!");
            payments.Exists(o => o.Type != PaymentType.DynamicSplit).ThrowIfTrue("请选择正确收款类型进行关联结算");

            var currentPayments = new List<Payment>();//不使用传入的List 防止修改了原List的数据
            currentPayments.AddRange(payments);

            var rtns = new List<Payment>();

            foreach (var settlement in settlements)
            {
                //说明已经建立关联关系的settlement
                if (settlement.AmountPayment >= settlement.Amount) continue;
                foreach (var payment in currentPayments)
                {
                    var settlementAmount = settlement.Amount - settlement.AmountPayment;
                    if (settlementAmount <= 0) break;
                    if (payment.Amount <= 0) continue;

                    var amount = settlementAmount <= payment.Amount ? settlementAmount : payment.Amount;

                    var payPayment = new Payment();
                    payPayment.PopulateInstance(payment);
                    payPayment.SerialNumber = string.Empty;
                    payPayment.Kvid = Guid.NewGuid();
                    payPayment.ParentKvid = payment.Kvid;
                    payPayment.BizId = settlement.SerialNumber;
                    payPayment.BizType = typeof(Settlement).FullName;
                    payPayment.BizKvid = settlement.Kvid;
                    payPayment.Type = PaymentType.Pay;
                    payPayment.Amount = amount;
                    payPayment.AmountUsed = amount;
                    payPayment.AmountSplited = amount;
                    rtns.Add(payPayment);

                    payment.Amount -= amount;
                    settlement.AmountPayment += amount;
                }
                settlement.AddOnlyProperties(o => o.AmountPayment);
            }

            return rtns;
        }

        /// <summary>
        /// 到账 支付或充值 调用方法
        /// </summary>
        /// <param name="payment"></param>
        /// <param name="payerAccount"></param>
        /// <param name="payeeAccount"></param>
        /// <returns></returns>
        internal static List<AccountDetail> Using(this Payment payment, Account payerAccount, Account payeeAccount, string remark = null)
        {
            payment.ThrowIfNull("付款记录不能为空!");
            payerAccount.ThrowIfNull("未找到付款账户!");
            payeeAccount.ThrowIfNull("未找到收款账户!");

            (payment.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (payment.Type == PaymentType.DynamicSplit || payment.Type == PaymentType.Transfer).ThrowIfFalse("此方法仅支持支付或充值");
            (payment.Amount <= 0).ThrowIfTrue("付款记录发生金额不能少于0!");
            (payment.PayerAccountKvid == Guid.Empty || payment.PayeeAccountKvid == Guid.Empty).ThrowIfTrue("收付款账户不能为空!");
            (payment.AmountUsed != payment.Amount).ThrowIfTrue("已使用金额不等于支付金额!");
            //(payment.PayerAccountKvid != payerAccount.Kvid).ThrowIfTrue("传入付款账户不一致!");
            //(payment.PayeeAccountKvid != payeeAccount.Kvid).ThrowIfTrue("传入收款账户不一致!");
            (payerAccount.Currency != payment.Currency).ThrowIfTrue($"付款账户({payerAccount.Currency})与此付款({payment.Currency})的货币单位不一致!");
            (payeeAccount.Currency != payment.Currency).ThrowIfTrue($"收款账户({payeeAccount.Currency})与此付款({payment.Currency})的货币单位不一致!");

            var rtns = new List<AccountDetail>();
            #region 付款账户变动
            var payerAccountDetail = new AccountDetail();
            payerAccountDetail.AccountKvid = payerAccount.Kvid;
            payerAccountDetail.BizId = payment.SerialNumber;
            payerAccountDetail.BizKvid = payment.Kvid;
            payerAccountDetail.BizType = typeof(Payment).FullName;

            payerAccountDetail.PayerKvid = payment.PayerKvid;
            payerAccountDetail.PayerName = payment.PayerName;
            payerAccountDetail.PayerAccountKvid = payerAccount.Kvid;
            payerAccountDetail.PayerAccountName = payerAccount.Name;
            payerAccountDetail.PayerAccountSerialNumber = payerAccount.SerialNumber;

            payerAccountDetail.PayeeKvid = payment.PayeeKvid;
            payerAccountDetail.PayeeName = payment.PayeeName;
            payerAccountDetail.PayeeAccountKvid = payeeAccount.Kvid;
            payerAccountDetail.PayeeAccountName = payeeAccount.Name;
            payerAccountDetail.PayeeAccountSerialNumber = payeeAccount.SerialNumber;

            payerAccountDetail.AmountPayment = payment.Amount;

            payerAccountDetail.Summary = payment.Summary;
            payerAccountDetail.Remark = remark;

            rtns.Add(payerAccountDetail);
            #endregion

            #region 收款账户变动
            var payeeAccountDetail = new AccountDetail();
            payeeAccountDetail.AccountKvid = payeeAccount.Kvid;
            payeeAccountDetail.BizId = payment.SerialNumber;
            payeeAccountDetail.BizKvid = payment.Kvid;
            payeeAccountDetail.BizType = typeof(Payment).FullName;

            payeeAccountDetail.PayerKvid = payment.PayerKvid;
            payeeAccountDetail.PayerName = payment.PayerName;
            payeeAccountDetail.PayerAccountKvid = payerAccount.Kvid;
            payeeAccountDetail.PayerAccountName = payerAccount.Name;
            payeeAccountDetail.PayerAccountSerialNumber = payerAccount.SerialNumber;

            payeeAccountDetail.PayeeKvid = payment.PayeeKvid;
            payeeAccountDetail.PayeeName = payment.PayeeName;
            payeeAccountDetail.PayeeAccountKvid = payeeAccount.Kvid;
            payeeAccountDetail.PayeeAccountName = payeeAccount.Name;
            payeeAccountDetail.PayeeAccountSerialNumber = payeeAccount.SerialNumber;

            payeeAccountDetail.AmountPayment = payment.Amount;

            payeeAccountDetail.Summary = payment.Summary;
            payeeAccountDetail.Remark = remark;

            rtns.Add(payeeAccountDetail);
            #endregion

            return rtns;
        }

    }
}
