﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "借票查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceDebitQuery : RestfulExecution<Invoice>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime? BeginTime { get; set; }
        public DateTime? EndTime { get; set; }

        public bool? Debited { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (BeginTime != null && EndTime != null && BeginTime.Value > EndTime.Value).ThrowIfTrue("开始日期不能大于结束时间!");
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Invoice>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && o.Category == InvoiceApplyType.Debit.ToString());
            if (BeginTime != null) sqlExpress.And(o => o.OperateTime >= BeginTime.Value);
            if (EndTime != null) sqlExpress.And(o => o.OperateTime < EndTime.Value);
            if (Debited != null)
            {
                if (Debited.Value) sqlExpress.And(o => o.AmountPayment == o.Amount);
                else sqlExpress.And(o => o.AmountPayment < o.Amount);
            }
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            if (Debited != null)
            {
                if (!Debited.Value)
                {
                    var queryBankAccount = conn.From<Account>();
                    queryBankAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Bank, AccountType.WeChat, AccountType.AliPay));
                    queryBankAccount.Select(o => o.Kvid);

                    var queryCashAccount = conn.From<Account>();
                    queryCashAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Cash, AccountType.Pos));
                    queryCashAccount.Select(o => o.Kvid);

                    var queryPayment = conn.From<Payment>();
                    queryPayment.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && o.AmountInvoice < o.Amount && (Sql.In(o.PayeeAccountKvid, queryBankAccount) || Sql.In(o.PayerAccountKvid, queryCashAccount)));
                    queryPayment.Select(o => o.PayerName);
                    var paymentNames = conn.Column<string>(queryPayment).Distinct().ToList();
                    foreach (var item in rtns.Results)
                    {
                        if (paymentNames.Exists(o => o == item.PayerName))
                        {
                            item.Type = "Debit";
                        }
                    }
                    rtns.Results = rtns.Results.OrderBy(o => o.Type).ToList();
                }
            }
            return rtns;
        }
    }

    [Api(Description = "借票冲销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceDebiting : RestfulExecution<Invoice>
    {
        public Guid ApplyKvid { get; set; }
        public Guid PaymentKvid { get; set; }
        //public AccountType Type { get; set; }
        public decimal Amount { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (ApplyKvid == Guid.Empty) throw new Exception("请传入借票申请对象!");
            //if (Type != AccountType.Bank && Type != AccountType.Cash && Type != AccountType.Pos) throw new Exception("冲销类型不正确!");
            //if (Type != AccountType.Bank && Amount <= 0) throw new Exception("冲销金额不能小于0!");
            //if (Type == AccountType.Bank && PaymentKvid == Guid.Empty) throw new Exception("请选择到账进行冲销!");

            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var apply = conn.SingleById<InvoiceApply>(ApplyKvid);
            apply.ThrowIfNull("未找到所选借票申请数据!");
            (apply.OperateType != InvoiceApplyType.Debit).ThrowIfTrue("非借票不可冲销!");
            (apply.OffsetKvid != Guid.Empty).ThrowIfTrue("当前借票申请已冲账,无法继续操作!");
            (apply.Status != (int)InvoiceApplyStatus.Completed).ThrowIfTrue("当前借票申请未执行完毕,无法冲销!");
            var existInvoices = conn.Select<Invoice>(o => o.ApplyKvid == apply.Kvid && o.OffsetKvid == Guid.Empty && o.RootKvid == o.Kvid);
            if (existInvoices.IsNullOrEmpty()) throw new Exception("未找到所选申请对应的发票信息!");
            var unDebitAmount = existInvoices.Sum(o => o.Amount) - existInvoices.Sum(o => o.AmountPayment);
            (unDebitAmount <= 0).ThrowIfTrue("此申请无需再冲销!");
            var applyRelations = conn.Select<InvoiceApply>(o => o.ParentKvid == apply.Kvid && o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty);

            var rtns = new RestfulUpdateResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var payment = conn.SingleById<Payment>(PaymentKvid);
            if (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("请选择正确银行到账进行冲销!");
            if (payment.AmountInvoice >= payment.Amount) throw new Exception("当前到账已开票,无法冲销当前借票!");
            var splitPayments = conn.Select<Payment>(o => o.ParentKvid == payment.Kvid && o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split);
            if (Amount <= 0 || Amount > (payment.Amount - payment.AmountInvoice)) Amount = payment.Amount - payment.AmountInvoice;

            decimal amount = Amount >= unDebitAmount ? unDebitAmount : Amount;

            var relation = new InvoiceApply();//apply关联Payment
            relation.PopulateWith(apply);
            relation.Kvid = Guid.NewGuid();
            relation.RootKvid = apply.Kvid;
            relation.ParentKvid = apply.Kvid;
            relation.SerialNumber = string.Empty;
            relation.OperateType = InvoiceApplyType.Related;
            relation.OwnerKvid = apply.OwnerKvid;
            relation.OwnerName = apply.OwnerName;
            relation.BizId = payment.SerialNumber;
            relation.BizKvid = payment.Kvid;
            relation.BizType = typeof(Payment).FullName;
            relation.Type = apply.OperateType.ToString();
            relation.Amount = amount;
            relation.OperatorName = KiviiContext.CurrentMember.FullName;
            relation.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            relation.AmountInvoice = amount;
            relation.AmountUsed = amount;

            apply.AmountUsed = applyRelations.Sum(o => o.Amount) + amount;//更新apply的值
            apply.AddOnlyProperties(o => o.AmountUsed);

            var payments = new List<Payment>();
            payments.Add(payment);
            var correlatedInvoices = existInvoices.Correlating(payments);//生成payment和invoice之间的关联,同事原Invoices也要更新相关数据

            #region 优化
            if (!splitPayments.IsNullOrEmpty())//要是所选Payment存在拆分项,满足条件就要更新AmountInvoice
            {
                decimal amountInvoice = 0;
                amountInvoice.PopulateWith(payment.AmountInvoice);// = payment.AmountInvoice;
                foreach (var item in splitPayments)
                {
                    if (amountInvoice <= 0) continue;
                    item.AmountInvoice = item.Amount >= amountInvoice ? amountInvoice : item.Amount;
                    item.AddOnlyProperties(o => o.AmountInvoice);
                    amountInvoice -= item.AmountInvoice;
                }
            }
            #endregion

            var trans = conn.OpenTransaction();
            try
            {
                conn.Insert(relation);
                conn.UpdateOnly(apply);
                conn.UpdateOnly(payment);
                if (!splitPayments.IsNullOrEmpty())
                {
                    foreach (var item in splitPayments)
                    {
                        if (item.OnlyProperties.IsNullOrEmpty()) continue;
                        conn.UpdateOnly(item);
                    }
                }
                foreach (var item in correlatedInvoices)
                {
                    item.SerialNumber = item.GetSubSerialNumber(conn);
                    conn.Insert(item);
                }
                foreach (var item in existInvoices)
                {
                    if (item.OnlyProperties.Count > 0)
                    {
                        conn.UpdateOnly(item);
                        rtns.Results.Add(item);
                    }
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "冲销撤销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceDebitCancel : RestfulExecution<Invoice>
    {
        public Guid ApplyKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (ApplyKvid == Guid.Empty) throw new Exception("请传入借票申请对象!");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var apply = conn.SingleById<InvoiceApply>(ApplyKvid);
            apply.ThrowIfNull("未找到所选发票申请信息!");
            (apply.OperateType != InvoiceApplyType.Debit).ThrowIfTrue("非借票不可撤销!");
            (apply.OffsetKvid != Guid.Empty).ThrowIfTrue("当前借票申请已冲账,无法继续操作!");
            (apply.Status != (int)InvoiceApplyStatus.Completed).ThrowIfTrue("当前借票申请未完成开票,无法继续操作!");
            var allInvoices = conn.Select<Invoice>(o => o.ApplyKvid == apply.Kvid && o.OffsetKvid == Guid.Empty);
            if (allInvoices.IsNullOrEmpty()) throw new Exception("未找到所选申请对应的发票信息!");
            var existInvoices = allInvoices.Where(o => o.RootKvid == o.Kvid).ToList();//发票信息
            if (existInvoices.IsNullOrEmpty()) throw new Exception("未找到所选申请对应的发票信息!");
            var relationInvoices = allInvoices.Where(o => o.Type == "Relation" && o.OffsetKvid == Guid.Empty).ToList();//关联Payment的发票数据
            var applyRelations = conn.Select<InvoiceApply>(o => o.ParentKvid == apply.Kvid && o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty);//关联Payment的申请数据
            if (applyRelations.IsNullOrEmpty()) throw new Exception("当前申请无需冲销撤销");
            var existPayments = conn.Select<Payment>(o => Sql.In(o.Kvid, applyRelations.ConvertAll(p => p.BizKvid)));//Type:Bank,Split,Pos,Cash,WeChat...
            if (existPayments.IsNullOrEmpty()) throw new Exception("当前申请未找到到账冲销,无需撤销!");

            List<Payment> parentOrNodePayments = new List<Payment>();//父级或子级payment也要更新AmountInvoice
            var existPaymentRootKvids = existPayments.Where(o => o.RootKvid != o.Kvid).ToList().ConvertAll(p => p.RootKvid);
            if (!existPaymentRootKvids.IsNullOrEmpty())
            {
                var parents = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Kvid, existPaymentRootKvids));
                if (!parents.IsNullOrEmpty()) parentOrNodePayments.AddRange(parents);
            }

            var nodes = conn.Select<Payment>(o => Sql.In(o.ParentKvid, existPayments.ConvertAll(p => p.Kvid)) && o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split);
            if (!nodes.IsNullOrEmpty()) parentOrNodePayments.AddRange(nodes);

            var rtns = new RestfulUpdateResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var trans = conn.OpenTransaction();

            try
            {
                //删除关联payment的申请数据
                var updateApplyRelations = conn.From<InvoiceApply>();
                updateApplyRelations = updateApplyRelations.Update(o => o.Status);
                updateApplyRelations = updateApplyRelations.Where(o => Sql.In(o.Kvid, applyRelations.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<InvoiceApply>(new InvoiceApply { Status = -1 }, updateApplyRelations);

                //删除关联Payment的发票数据
                if (!relationInvoices.IsNullOrEmpty())
                {
                    var updateRelationInvoices = conn.From<Invoice>();
                    updateRelationInvoices = updateRelationInvoices.Update(o => o.Status);
                    updateRelationInvoices = updateRelationInvoices.Where(o => Sql.In(o.Kvid, relationInvoices.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<Invoice>(new Invoice { Status = -1 }, updateRelationInvoices);
                }
                //更新申请的到账金额
                apply.AmountUsed = 0;
                apply.AddOnlyProperties(o => o.AmountUsed);
                conn.UpdateOnly(apply);

                //更新发票的付款时间和到账金额
                var updateExistInvoices = conn.From<Invoice>();
                updateExistInvoices = updateExistInvoices.Update(o => new { o.PayedTime, o.AmountPayment });
                updateExistInvoices = updateExistInvoices.Where(o => Sql.In(o.Kvid, existInvoices.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Invoice>(new Invoice { PayedTime = null, AmountPayment = 0 }, updateExistInvoices);

                //更新已关联的payment的AmountInvoice
                var updateExistPayments = conn.From<Payment>();
                updateExistPayments = updateExistPayments.Update(o => o.AmountInvoice);
                updateExistPayments = updateExistPayments.Where(o => Sql.In(o.Kvid, existPayments.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Payment>(new Payment { AmountInvoice = 0 }, updateExistPayments);

                if (!parentOrNodePayments.IsNullOrEmpty())
                {
                    //更新已关联的payment的父级或子级AmountInvoice
                    var updateParentOrNodePayments = conn.From<Payment>();
                    updateParentOrNodePayments = updateParentOrNodePayments.Update(o => o.AmountInvoice);
                    updateParentOrNodePayments = updateParentOrNodePayments.Where(o => Sql.In(o.Kvid, parentOrNodePayments.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<Payment>(new Payment { AmountInvoice = 0 }, updateParentOrNodePayments);
                }

                rtns.Results.AddRange(existInvoices);
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
