﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "生成到账流水")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentAccept : RestfulExecution<Payment>
    {
        public Payment Item { get; set; }

        public List<Payment> Items { get; set; }

        [ApiMember(Description = "认领类型,默认为Department,Organization,Member")]
        public OwnerType OwnerType { get; set; } = OwnerType.Department;

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");
            if (Items.IsNullOrEmpty()) Items = new List<Payment>();
            if (Item != null) Items.Add(Item);
            if (Items.Exists(o => o.Currency != CurrencyUnit.CNY)) throw new Exception("暂仅支持生成人民币货币单位流水!");
            var typeBanks = Items.Where(o => o.Type == PaymentType.Bank).ToList();
            var typeCashs = Items.Where(o => o.Type == PaymentType.Cash).ToList();
            var typePoss = Items.Where(o => o.Type == PaymentType.Pos).ToList();

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var accountBalance = CurrencyUnit.CNY.GetBalanceAccount();

            var allAccountDetails = new List<AccountDetail>();
            var allAcceptPayments = new List<Payment>();

            //生成银行流水
            if (!typeBanks.IsNullOrEmpty())
            {
                var bankAccounts = conn.Select<Account>(o => o.Type == AccountType.Bank && o.Currency == CurrencyUnit.CNY && o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid);
                if (bankAccounts.IsNullOrEmpty()) throw new Exception("未找到当前登录人所在组织的银行账户!");
                foreach (var item in typeBanks)
                {
                    (item.PayeeAccountSerialNumber.IsNullOrEmpty()).ThrowIfTrue("银行流水收款单位PayeeAccountSerialNumber不能为空");
                    var bankAccount = bankAccounts.FirstOrDefault(o => o.SerialNumber == item.PayeeAccountSerialNumber);
                    if (bankAccount == null) throw new Exception($"未找到当前银行流水{item.PayerName}[{item.PayeeAccountSerialNumber}]对应系统银行账户!");

                    if (item.Amount <= 0) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = 0;
                    item.AmountUsed = 0;

                    item.OwnerKvid = KiviiContext.CurrentMember.OrganizationKvid;
                    item.OwnerName = KiviiContext.CurrentMember.OrganizationName;
                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = accountBalance.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? accountBalance.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? accountBalance.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? accountBalance.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = bankAccount.Kvid;
                    item.PayeeAccountName = bankAccount.Name;
                    item.PayeeAccountOwnerName = bankAccount.OwnerName;
                    item.PayeeAccountSerialNumber = bankAccount.SerialNumber;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetails = item.Accept();
                    allAcceptPayments.Add(item);
                    allAccountDetails.AddRange(accountDetails);
                }
            }

            var ownerKvid = Guid.Empty;
            var ownerName = string.Empty;
            switch (OwnerType)
            {
                case OwnerType.Organization:
                    {
                        ownerKvid = KiviiContext.CurrentMember.OrganizationKvid;
                        ownerName = KiviiContext.CurrentMember.OrganizationName;
                        break;
                    }
                case OwnerType.Member:
                    {
                        ownerKvid = KiviiContext.CurrentMember.Kvid;
                        ownerName = KiviiContext.CurrentMember.FullName;
                        break;
                    }
                default:
                    {
                        ownerKvid = KiviiContext.CurrentMember.DepartmentKvid;
                        ownerName = KiviiContext.CurrentMember.DepartmentName;
                        break;
                    }
            }

            //生成现金流水
            if (!typeCashs.IsNullOrEmpty())
            {
                var cashAccount = CurrencyUnit.CNY.GetPersonalAccount(AccountType.Cash);
                if (cashAccount == null) throw new Exception("未找到当前登录人的现金账户!");
                foreach (var item in typeCashs)
                {
                    if (item.Amount <= 0) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = item.Amount;
                    item.AmountUsed = 0;

                    item.OwnerKvid = ownerKvid;
                    item.OwnerName = ownerName;
                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = cashAccount.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? cashAccount.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? cashAccount.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? cashAccount.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = accountBalance.Kvid;
                    item.PayeeAccountName = accountBalance.Name;
                    item.PayeeAccountOwnerName = accountBalance.OwnerName;
                    item.PayeeAccountSerialNumber = accountBalance.SerialNumber;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetails = item.Accept();
                    allAcceptPayments.Add(item);
                    allAccountDetails.AddRange(accountDetails);
                }
            }
            //生成Pos流水
            if (!typePoss.IsNullOrEmpty())
            {
                var posAccount = CurrencyUnit.CNY.GetPersonalAccount(AccountType.Pos);
                if (posAccount == null) throw new Exception("未找到当前登录人的刷卡账户!");
                foreach (var item in typePoss)
                {
                    if (item.Amount <= 0) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = item.Amount;
                    item.AmountUsed = 0;

                    item.OwnerKvid = ownerKvid;
                    item.OwnerName = ownerName;
                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = posAccount.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? posAccount.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? posAccount.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? posAccount.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = accountBalance.Kvid;
                    item.PayeeAccountName = accountBalance.Name;
                    item.PayeeAccountOwnerName = accountBalance.OwnerName;
                    item.PayeeAccountSerialNumber = accountBalance.SerialNumber;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetails = item.Accept();
                    allAcceptPayments.Add(item);
                    allAccountDetails.AddRange(accountDetails);
                }
            }

            var rtns = new RestfulCreateResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();

            try
            {
                if (!allAcceptPayments.IsNullOrEmpty())
                {
                    foreach (var item in allAcceptPayments)
                    {
                        conn.Insert(item);
                        rtns.Results.Add(item);
                    }
                }
                if (!allAccountDetails.IsNullOrEmpty()) allAccountDetails.ForEach(o => o.Insert(conn));
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
