﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    /// <summary>
    /// 此接口用于以下情况
    /// 1.到账充值到客户存款账户
    /// 2.退款存入退款账户
    /// 3.非业务收入存入个人现金,刷卡等账户
    /// </summary>
    [Api(Description = "收款转账")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class PaymentTransfer : RestfulExecution<Payment>
    {
        public Guid AccountKvid { get; set; }

        /// <summary>
        /// 类型:Split,UnBiz,Refund,Cash,Pos,AliPay,Wechat
        /// </summary>
        public Guid PaymentKvid { get; set; }

        public decimal Amount { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (AccountKvid == Guid.Empty || PaymentKvid == Guid.Empty) throw new Exception("请选择要处理的到账信息!");
            if (Amount <= 0) throw new Exception("入账金额不能小于等0!");

            var conn = KiviiContext.GetOpenedDbConnection<Account>();
            var payment = conn.SingleById<Payment>(PaymentKvid);
            if (payment == null) throw new Exception("不存在的收款信息!");
            if (payment.OffsetKvid != Guid.Empty) throw new Exception("此收款已冲账!");
            if (payment.Type != PaymentType.UnBiz && payment.Type != PaymentType.Refund && payment.Type != PaymentType.Split && payment.Type != PaymentType.Cash && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat && payment.Type != PaymentType.Pos) throw new Exception("不支持收款充值操作类型!");
            if (payment.Amount - payment.AmountUsed < Amount) throw new Exception($"此到账可使用金额不足,仅剩:{payment.Amount - payment.AmountUsed}元");

            var accountPayee = conn.SingleById<Account>(AccountKvid);
            if (accountPayee == null) throw new Exception("不存在此账户");
            if (accountPayee.Type != AccountType.Cash && accountPayee.Type != AccountType.Pos && accountPayee.Type != AccountType.Deposit && accountPayee.Type != AccountType.Refund) throw new Exception("不支持的账户类型");

            var accountPayer = accountPayee.Currency.GetBalanceAccount();
            accountPayer.ThrowIfNull($"未能正确获取系统{accountPayee.Currency}平衡账户!请联系管理员!");

            //生成转账Payment
            Payment transfer = null;
            //生成账户明细
            List<AccountDetail> accountDetails = new List<AccountDetail>();
            //存在的父级Payment要更新AmountUsed
            Payment parentPayment = null;

            //3.
            if (payment.Type == PaymentType.UnBiz)
            {
                if (accountPayee.Type != AccountType.Cash && accountPayee.Type != AccountType.Pos) throw new Exception("非业务收款,仅支持现金或刷卡个人账户充值!");
                transfer = payment.Transfer(accountPayee, accountPayer, Amount, $"非业务入账{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}");
                accountDetails = transfer.Using(accountPayer, accountPayee);
            }
            //2.
            else if (payment.Type == PaymentType.Refund)
            {
                if (accountPayee.Type != AccountType.Refund) throw new Exception("退款收款,仅支持退款账户充值!");
                transfer = payment.Transfer(accountPayee, accountPayer, Amount, $"退款入账{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}");
                accountDetails = transfer.Using(accountPayer, accountPayee);
            }
            //1.
            else
            {
                if(accountPayee.Type!=AccountType.Deposit) throw new Exception("业务收款,仅支持客户存款账户充值!");
                transfer = payment.Transfer(accountPayee, accountPayer, Amount, $"到账充值{(Remark.IsNullOrEmpty() ? string.Empty : ":" + Remark)}");
                accountDetails = transfer.Using(accountPayer, accountPayee);
            }
            if (payment.ParentKvid != Guid.Empty) parentPayment = conn.SingleById<Payment>(payment.ParentKvid);

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                if (transfer != null)
                {
                    conn.Insert(transfer);
                    rtns.Results.Add(transfer);
                }
                accountDetails.ForEach(o => o.Insert(conn));
                conn.UpdateOnly(payment);
                if (parentPayment != null)
                {
                    parentPayment.AmountUsed += transfer.Amount;
                    parentPayment.AddOnlyProperties(o => o.AmountUsed);
                    conn.UpdateOnly(parentPayment);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    //(重点测试下2021.09.14)
    [Api(Description = "收款转账撤销")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class PaymentTransferCancel : RestfulExecution<Payment>
    {
        /// <summary>
        /// 类型:Transfer
        /// </summary>
        public List<Guid> PaymentKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            PaymentKvids.ThrowIfNullOrEmpty("请传入要撤销转账的内容!");
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(PaymentKvids);
            payments.ThrowIfNullOrEmpty("未找到要撤销转账的信息!");
            (payments.Count != PaymentKvids.Count).ThrowIfTrue("所选到账与查询到数据数量不符合!");
            payments.Exists(o => o.Type != PaymentType.Transfer).ThrowIfTrue("存在不正确的到账类型!");
            payments.Exists(o => o.OffsetKvid != Guid.Empty).ThrowIfTrue("存在已撤销的转账!");
            payments.Exists(o => o.ParentKvid == Guid.Empty).ThrowIfTrue("存在不完整的转账数据!");

            var parentPayments = conn.SelectByIds<Payment>(payments.ConvertAll(o => o.ParentKvid));
            parentPayments.ThrowIfNullOrEmpty("未找到夫级付款,数据错误!");
            var rootPayments = conn.SelectByIds<Payment>(parentPayments.ConvertAll(o => o.ParentKvid));

            var accountPayees = conn.SelectByIds<Account>(payments.ConvertAll(o => o.PayeeAccountKvid));
            accountPayees.ThrowIfNullOrEmpty("未找到收款账户!");
            var accountPayers = conn.SelectByIds<Account>(payments.ConvertAll(o => o.PayerAccountKvid));
            accountPayers.ThrowIfNullOrEmpty("未找到付款账户!");

            //新生成的Offset数据
            var offsets = new List<Payment>();
            //原Transfer数据更新Offset
            var orginOffsets = new List<Payment>();
            //生成的账户明细
            var totalAccountDetails = new List<AccountDetail>();
            //Parent和Root 更新AmountUsed
            var roots = new List<Payment>();

            foreach (var item in payments)
            {
                var parent = parentPayments.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                parent.ThrowIfNull($"未找到{item.SerialNumber}当前的到账上级节点!");
                var root = rootPayments.FirstOrDefault(o => o.Kvid == parent.ParentKvid);
                var accountPayee = accountPayees.FirstOrDefault(o => o.Kvid == item.PayeeAccountKvid);
                accountPayee.ThrowIfNull($"未找到{item.SerialNumber}对应收款账户!");
                var accountPayer = accountPayers.FirstOrDefault(o => o.Kvid == item.PayerAccountKvid);
                accountPayer.ThrowIfNull($"未找到{item.SerialNumber}对应付款账户!");

                #region 新增流水
                var payeeAaccountDetail = new AccountDetail();
                payeeAaccountDetail.AccountKvid = accountPayee.Kvid;
                payeeAaccountDetail.BizId = item.SerialNumber;
                payeeAaccountDetail.BizKvid = item.Kvid;
                payeeAaccountDetail.BizType = typeof(Payment).FullName;

                payeeAaccountDetail.PayerKvid = item.PayeeKvid;
                payeeAaccountDetail.PayerName = item.PayeeName;
                payeeAaccountDetail.PayerAccountKvid = accountPayee.Kvid;
                payeeAaccountDetail.PayerAccountName = accountPayee.Name;
                payeeAaccountDetail.PayerAccountSerialNumber = accountPayee.SerialNumber;

                payeeAaccountDetail.PayeeKvid = item.PayerKvid;
                payeeAaccountDetail.PayeeName = item.PayerName;
                payeeAaccountDetail.PayeeAccountKvid = accountPayer.Kvid;
                payeeAaccountDetail.PayeeAccountName = accountPayer.Name;
                payeeAaccountDetail.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                payeeAaccountDetail.AmountPayment = item.Amount;
                payeeAaccountDetail.Summary = "撤销转账";
                #endregion
                totalAccountDetails.Add(payeeAaccountDetail);

                #region 新增流水
                var payerAaccountDetail = new AccountDetail();
                payerAaccountDetail.AccountKvid = accountPayer.Kvid;
                payerAaccountDetail.BizId = item.SerialNumber;
                payerAaccountDetail.BizKvid = item.Kvid;
                payerAaccountDetail.BizType = typeof(Payment).FullName;

                payerAaccountDetail.PayerKvid = item.PayeeKvid;
                payerAaccountDetail.PayerName = item.PayeeName;
                payerAaccountDetail.PayerAccountKvid = accountPayee.Kvid;
                payerAaccountDetail.PayerAccountName = accountPayee.Name;
                payerAaccountDetail.PayerAccountSerialNumber = accountPayee.SerialNumber;

                payerAaccountDetail.PayeeKvid = item.PayerKvid;
                payerAaccountDetail.PayeeName = item.PayerName;
                payerAaccountDetail.PayeeAccountKvid = accountPayer.Kvid;
                payerAaccountDetail.PayeeAccountName = accountPayer.Name;
                payerAaccountDetail.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                payerAaccountDetail.AmountPayment = item.Amount;
                payerAaccountDetail.Summary = "撤销转账";
                #endregion
                totalAccountDetails.Add(payerAaccountDetail);

                var offset = item.Offset($"撤销转账");
                offsets.Add(offset);
                orginOffsets.Add(item);

                if (roots.Exists(o => o.Kvid == parent.Kvid))
                {
                    var exist = roots.FirstOrDefault(o => o.Kvid == parent.Kvid);
                    exist.AmountUsed -= item.Amount;
                }
                else
                {
                    parent.AmountUsed -= item.Amount;
                    parent.AddOnlyProperties(o => o.AmountUsed);
                    roots.Add(parent);
                }
                if (root != null)
                {
                    if (roots.Exists(o => o.Kvid == root.Kvid))
                    {
                        var exist = roots.FirstOrDefault(o => o.Kvid == root.Kvid);
                        exist.AmountUsed -= item.Amount;
                    }
                    else
                    {
                        root.AmountUsed -= item.Amount;
                        root.AddOnlyProperties(o => o.AmountUsed);
                        roots.Add(root);
                    }
                }
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();

            try
            {
                foreach(var offset in offsets)
                {
                    conn.Insert(offset);
                }

                foreach (var item in orginOffsets)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }

                foreach(var  item in totalAccountDetails)
                {
                    item.Insert(conn);
                }

                foreach( var item in roots)
                {
                    conn.UpdateOnly(item);
                }

                trans.Commit();
            }
            catch(Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
